# frequencywidget.py -- widget used in the frequency tab
#
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2013
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

from math import sin, pi
import numpy
from PyQt4 import QtCore, QtGui
from cclib.bridge import makeopenbabel

from ui_frequency_widget import Ui_frequencyWidget

NLOOPS = 10.0
labels = ["Frequency", "IR Intensity", "Raman Intensity"]

class FrequencyWidget(QtGui.QWidget, Ui_frequencyWidget):
    def __init__(self, parent=None):
        super(FrequencyWidget, self).__init__(parent)

        self.setupUi(self)
        self.interval = 100
        self.value = 0.0
        self.displacement = 0.5
        self.index = 0

        self.obmols = {}

    def setData(self, data):
        self.data = data
        self.model = FrequencyModel(data, self.frequencyView)
        self.frequencyView.setModel(self.model)
        self.selection = QtGui.QItemSelectionModel(self.model)
        self.frequencyView.setSelectionModel(self.selection)
        self.frequencyView.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.frequencyView.resizeColumnsToContents()

        QtCore.QObject.connect(self.selection, 
                                QtCore.SIGNAL("selectionChanged( const QItemSelection, \
                                const QItemSelection)"), 
                                self.changeSelection)

        self.glWidget.setMolecule(self.data.atomcoords[-1], self.data.atomnos)

    def beginAnimation(self):
        self.timer = self.startTimer(self.interval)
        self.value = 0.0

    def endAnimation(self):
        if hasattr(self, "timer") and self.timer:
            self.killTimer(self.timer)

    def changeSelection(self, selected, deselected):
        indexes = selected.indexes()
        
        if len(indexes) == 3: # assume only one row selected
            self.index = indexes[0].row()
            self.obmols = {}

    def timerEvent(self, event):
        if not hasattr(self, "data"):
            return

        self.value += 1
        if self.value >= NLOOPS:
            self.value = 0.0

        value = sin(self.value * 2 * pi / NLOOPS)
        str_value = "%.3f" % value
        if not str_value in self.obmols:
            newcoords = self.data.atomcoords[-1] + \
                            self.displacement * value * self.data.vibdisps[self.index]
            self.obmols[str_value] = makeopenbabel(newcoords, self.data.atomnos)

        self.glWidget.obmol = self.obmols[str_value]
        self.glWidget.updateGL()

    def save(self, filename):
        
        if hasattr(self.data, "vibfreqs"):
            header = "Freqs"
            format = "%7.2f"
            results = self.data.vibfreqs
        else:
            return

        if hasattr(self.data, "vibirs"):
            header += "\tIR"
            format += "\t%9.3f"
            results = numpy.column_stack([results, self.data.vibirs])

        if hasattr(self.data, "vibramans"):
            header += "\tRaman"
            format += "\t%9.3f"
            results = numpy.column_stack([results, self.data.vibramans])

        # check to see if we have a name
        if filename:
            fileinfo = QtCore.QFileInfo(filename)
            ext = "." + fileinfo.suffix()
            rindex = str(filename).rfind(ext)
            newfilename = filename[:rindex] + "-freq.txt"

            filename = QtGui.QFileDialog.getSaveFileName(self, "Save frequency results...", \
                                    newfilename, "*.txt")

        else:
            newfilename = "newfile-freq.txt"
            filename = QtGui.QFileDialog.getSaveFileName(self, "Save frequency results...", \
                                    newfilename, "*.txt")

        if not filename:
            return

        file = open(str(filename), "w")
        file.write(header)
        file.write("\n")
        numpy.savetxt(file, results, delimiter="\t", fmt=format)
        file.close()

class FrequencyModel(QtCore.QAbstractTableModel):
    def __init__(self, data, parent=None):
        super(FrequencyModel, self).__init__(parent)

        self.data = data

    def headerData(self, section, orientation, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return labels[section]

        if orientation == QtCore.Qt.Vertical:
            return section + 1

    def rowCount(self, parent):
        if not hasattr(self.data, "vibfreqs"):
            return 0

        return len(self.data.vibfreqs)

    def columnCount(self, parent):
        return 3

    def data(self, index, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if not self.data:
            return None

        if index.column() == 0 and hasattr(self.data, "vibfreqs"):
            value = self.data.vibfreqs[index.row()]
            return "%i" % value
        if index.column() == 1 and hasattr(self.data, "vibirs"):
            value = self.data.vibirs[index.row()]
            return "%.2f" % value
        if index.column() == 2 and hasattr(self.data, "vibramans"):
            value = self.data.vibramans[index.row()]
            return "%.2f" % value

        return None

if __name__ == "__main__":
    import sys
    from cclib.parser import ccopen

    parser = ccopen(sys.argv[1])
    data = parser.parse()
    app = QtGui.QApplication(sys.argv)
    widget = FrequencyWidget()
    widget.show()
    widget.beginAnimation()
    widget.setData(data)

    result = app.exec_()
    sys.exit(result)

